@@ -1,7 +1,7 @@
 #
 # facade.rb
 #
-#   Copyright (c) 1998-2001 Minero Aoki <aamine@loveruby.net>
+#   Copyright (c) 1998-2002 Minero Aoki <aamine@loveruby.net>
 #
 #   This program is free software.
 #   You can distribute/modify this program under the terms of
@@ -19,6 +19,7 @@ module TMail
 
     class << self
     
+     #RPP tmail_attr_reader
       def tmail_attr_reader( header, name, getter )
         module_eval s = <<-"}", __FILE__, __LINE__ + 1
           def #{name.id2name}( default = nil )
@@ -31,7 +32,8 @@ module TMail
         }
       end
 
-      def tmail_attr_writer( header, name, setter = 'adds.replace' )
+     #RPP tmail_attr_writer
+      def tmail_attr_writer( header, name, setter = 'addrs.replace' )
         module_eval <<-"}", __FILE__, __LINE__ + 1
           def #{name.id2name}=( *arr )
             store '#{header}', ''
@@ -40,38 +42,28 @@ module TMail
           end
         }
       end
-
-      def tmail_s_attr_writer( header, name = nil )
-        name = name ? name.id2name : header.downcase.gsub(/-/, '_')
-        module_eval <<-"}", __FILE__, __LINE__ + 1
-          def #{name}=( str )
-            if str then
-              store '#{header}', str
-            else
-              @header.delete '#{header.downcase}'
-            end
-            str
-          end
-        }
-      end
-
-      def tmail_sa_attr_writer( header )
-        name = header.downcase.gsub(/-/, '_')
-        module_eval <<-"}", __FILE__, __LINE__ + 1
-          def #{name}=( *strs )
-            strs.flatten!
-            if strs.empty? then
-              @header.delte '#{header.downcase}'
-            else
-              store '#{header}', strs.join(', ')
-            end
-            strs
-          end
-        }
-      end
     
     end
 
+    def set_string_array_attr( key, strs )
+      strs.flatten!
+      if strs.empty? then
+        @header.delete header.downcase
+      else
+        store key, strs.join(', ')
+      end
+      strs
+    end
+
+    def set_string_attr( key, str )
+      if str then
+        store key, str
+      else
+        @header.delete key.downcase
+      end
+      str
+    end
+
 
     #
     # date time
@@ -111,13 +103,31 @@ module TMail
     tmail_attr_writer 'Bcc', :bcc_addrs
 
     def to( default = nil )
-      addr, = to_addrs(nil)
-      addr || default
+      arr = to_addrs(nil) or return default
+      arr.collect {|a| a.spec }
     end
 
-    tmail_sa_attr_writer 'To'
-    tmail_sa_attr_writer 'Cc'
-    tmail_sa_attr_writer 'Bcc'
+    def cc( default = nil )
+      arr = cc_addrs(nil) or return default
+      arr.collect {|a| a.spec }
+    end
+
+    def bcc( default = nil )
+      arr = bcc_addrs(nil) or return default
+      arr.collect {|a| a.spec }
+    end
+
+    def to=( *strs )
+      set_string_array_attr 'To', strs
+    end
+
+    def cc=( *strs )
+      set_string_array_attr 'Cc', strs
+    end
+
+    def bcc=( *strs )
+      set_string_array_attr 'Bcc', strs
+    end
 
 
     #
@@ -128,14 +138,19 @@ module TMail
     tmail_attr_writer 'From', :from_addrs
 
     def from( default = nil )
-      if a = from_addr then
-        a.phrase || a.address
-      else
-        default
-      end
+      arr = from_addrs(nil) or return default
+      arr.collect {|a| a.spec }
     end
 
-    tmail_sa_attr_writer 'From'
+    def from=( *strs )
+      set_string_array_attr 'From', strs
+    end
+
+    def friendly_from( default = nil )
+      a, = from_addrs(nil)
+      a or return default
+      a.phrase || a.spec
+    end
 
     # obsolete
     def from_addr( default = nil )
@@ -146,7 +161,7 @@ module TMail
     # obsolete
     def from_address( default = nil )
       if a = from_addr(nil) then
-        a.address
+        a.spec
       else
         default
       end
@@ -167,21 +182,42 @@ module TMail
 
     tmail_attr_reader 'Reply-To', :reply_to_addrs, 'addrs'
     tmail_attr_writer 'Reply-To', :reply_to_addrs
-    tmail_sa_attr_writer 'Reply-To'
 
-
-    tmail_attr_reader 'Sender', :sender, 'addr'
-
-    def sender( default )
-      if f = @header['sender'] then
-        if a = f.addr then
-          return a.address
-        end
-      end
-      default
+    def reply_to( default = nil )
+      arr = reply_to_addrs(nil) or return default
+      arr.collect {|a| a.spec }
     end
 
-    tmail_s_attr_writer 'Sender'
+    def reply_to=( *strs )
+      set_string_array_attr 'Reply-To', strs
+    end
+
+
+    def sender_addr( default = nil )
+      f = @header['sender'] or return default
+      f.addr                or return default
+      f.addr
+    end
+
+    def sender_addr=( addr )
+      if addr then
+        store 'Sender', ''
+        @header['sender'].addr = addr
+      else
+        @header.delete 'sender'
+      end
+      addr
+    end
+
+    def sender( default )
+      f = @header['sender'] or return default
+      a = f.addr            or return default
+      a.spec
+    end
+
+    def sender=( str )
+      set_string_attr 'Sender', str
+    end
 
 
     #
@@ -189,24 +225,36 @@ module TMail
     #
 
     tmail_attr_reader 'Subject', :subject, 'body'
-    tmail_s_attr_writer 'Subject'
+
+    def subject=( str )
+      set_string_attr 'Subject', str
+    end
 
 
     #
     # identity & threading
     #
 
-    tmail_attr_reader 'Message-Id', :message_id, 'msgid'
-    tmail_s_attr_writer 'Message-Id'
+    tmail_attr_reader 'Message-Id', :message_id, 'id'
+
+    def message_id=( str )
+      set_string_attr 'Message-Id', str
+    end
 
     alias msgid  message_id
     alias msgid= message_id=
 
-    tmail_attr_reader 'In-Reply-To', :in_reply_to, 'msgids'
-    tmail_sa_attr_writer 'In-Reply-To'
+    tmail_attr_reader 'In-Reply-To', :in_reply_to, 'ids'
+
+    def in_reply_to=( *idstrs )
+      set_string_array_attr 'In-Reply-To', idstrs
+    end
 
     tmail_attr_reader 'References', :references, 'refs'
-    tmail_sa_attr_writer 'References'
+
+    def references=( *strs )
+      set_string_array_attr 'References', strs
+    end
 
 
     #
@@ -230,8 +278,8 @@ module TMail
     end
 
 
-    tmail_attr_reader 'Content-Type', :main_type, 'main'
-    tmail_attr_reader 'Content-Type', :sub_type,  'sub'
+    tmail_attr_reader 'Content-Type', :main_type, 'main_type'
+    tmail_attr_reader 'Content-Type', :sub_type,  'sub_type'
 
     def set_content_type( str, sub = nil, param = nil )
       if sub then
@@ -240,8 +288,8 @@ module TMail
         main, sub = str.split('/', 2)
       end
       if f = @header['content-type'] then
-        f.main = main
-        f.sub  = sub
+        f.main_type = main
+        f.sub_type  = sub
       else
         store 'Content-Type', "#{main}/#{sub}"
       end
@@ -252,12 +300,12 @@ module TMail
 
     alias content_type= set_content_type
 
-    tmail_attr_reader 'Content-Type', :charset, "params['charset']"
+    tmail_attr_reader 'Content-Type', :charset, "[]('charset')"
 
     def charset=( str )
       if str then
         if f = @header[ 'content-type' ] then
-          f.params['charset'] = str
+          f['charset'] = str
         else
           store 'Content-Type', "text/plain; charset=#{str}"
         end
@@ -267,7 +315,11 @@ module TMail
 
 
     tmail_attr_reader 'Content-Transfer-Encoding', :transfer_encoding, 'encoding'
-    tmail_s_attr_writer 'Content-Transfer-Encoding', :transfer_encoding
+
+    def transfer_encoding=( str )
+      set_string_attr 'Content-Transfer-Encoding', str
+    end
+
     alias encoding                   transfer_encoding
     alias encoding=                  transfer_encoding=
     alias content_transfer_encoding  transfer_encoding
@@ -275,9 +327,6 @@ module TMail
 
 
     tmail_attr_reader 'Content-Disposition', :disposition, 'disposition'
-    tmail_s_attr_writer 'Content-Disposition', :disposition
-    alias content_disposition  disposition
-    alias content_disposition= disposition=
 
     def set_content_disposition( str, params = nil )
       if f = @header['content-disposition'] then
@@ -288,6 +337,10 @@ module TMail
       f.params.replace params if params
     end
 
+    alias content_disposition  disposition
+    alias content_disposition= set_content_disposition
+    alias disposition=         set_content_disposition
+
 
     ###
     ### utils
@@ -312,7 +365,7 @@ module TMail
     def base64_decode
       if /base64/i === (@header['content-transfer-encoding'] || '') then
         store 'Content-Transfer-Encoding', '8bit'
-        self.body = Base64.decode(self.body)
+        self.body = Base64.decode(self.body, @config.strict_base64decode?)
       end
     end
 
